#[repr(transparent)]pub struct Guarded<T> {
pub borrow: T,
/* private fields */
}Expand description
A borrow guarded by an invariant.
This is used to define GuardedBorrow.
This can also be used to define the equivalent of GuardedBorrow with
smart pointers, like RefMut.
Fields
borrow: TBorrow contained in this guard.
The T type parameter is meant to be a mutable borrow type (like
&mut T, or RefMut<T>).
Implementations
Sourceimpl<'a, T: ?Sized> Guarded<&'a mut T>
impl<'a, T: ?Sized> Guarded<&'a mut T>
Sourcepub fn new(borrow: &'a mut T, guard: Snapshot<Mapping<T, bool>>) -> Self
pub fn new(borrow: &'a mut T, guard: Snapshot<Mapping<T, bool>>) -> Self
Create a new guarded borrow.
The borrow contained in the result is guaranteed to satisfy the
guard at the end of its lifetime.
Note that the borrow field cannot have its final value changed,
ensuring that it will not get swapped for another borrow.
ensures
result.borrow == borrowensures
forall<bor: &mut T> result.guard()[bor] == (guard[*bor] && ^bor == ^borrow)
ensures
guard[^borrow]ghost
Get a shared borrow out of this guarded borrow.
It is not possible to break the guard anymore, since the returned borrow is immutable.
ensures
*result == *self.borrowensures
*self.borrow == ^self.borrowghost